@@ -1,8 +1,7 @@
-# -*- coding: UTF-8 -*-
 #
 #      core.py
 #
-#      Copyright 2009-2012
+#      Copyright 2009-2020
 #      Giuseppe Penone <giuspen@gmail.com>,
 #      Chris Camacho (chris_c) <codifies@gmail.com>.
 #
@@ -24,8 +23,8 @@
 #      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #      MA 02110-1301, USA.
 
-import support, cons
-
+import support
+import cons
 
 def tile_vertically(windows_list, monitors_areas, dws):
     """Tile the Checked Windows Vertically"""
